#! /usr/bin/python
# -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
# vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
#
# Copyright (C) 2009 Sun Microsystems
#
# Authors:
#
#  Jay Pipes <joinfu@sun.com>
#
# This file is part of the Drizzle Automation Project.
#
# The Drizzle Automation Project is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# The Drizzle Automation Project is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with The Drizzle Automation Project (see COPYING.LESSER). If not, 
# see <http://www.gnu.org/licenses/>.

import commands
from drizzle.automation.lib  import logging
import sys
import os
import random

# Local variables
_mysql_user= ""
_mysql_pass= ""
_mysql_db_name= ""
_mysql_host= ""

def init(variables):
  global _mysql_user
  global _mysql_pass
  global _mysql_db_name
  global _mysql_host
  try:
    _mysql_user= variables['db']['mysql_user']
    _mysql_pass= variables['db']['mysql_pass']
    _mysql_db_name= variables['db']['mysql_db_name']
    _mysql_host= variables['db']['mysql_host']
  except:
    logging.error("Failed to find the required stats section in a configuration file.")
    sys.exit(1)

def create_db():
  global _mysql_user
  global _mysql_pass
  global _mysql_db_name
  global _mysql_host
  # Try finding the stats database in the mysql server
  (retcode, output)= commands.getstatusoutput("mysql --user=%s --password=%s --host=%s mysql -e\"SHOW DATABASES\"" % (_mysql_user, _mysql_pass, _mysql_host))

  if not retcode == 0:
    logging.error("Failed to connect to MySQL. Got error: %s" % output)
    sys.exit(1)
  else:
    if not _mysql_db_name in output:
      init_sql= """
CREATE DATABASE %s;
GRANT ALL ON %s TO %s@%s;
""" % (_mysql_db_name, _mysql_db_name, _mysql_user, _mysql_host)

      logging.info("Initializing database.")
      
      init_db_filename= "tmpinitdbfile.sql"
      f= open(init_db_filename, 'w')
      f.write(init_sql)
      f.close()

      (retcode, output)= commands.getstatusoutput("mysql --user=%s --password=%s --host=%s mysql < %s" % (_mysql_user, _mysql_pass, _mysql_host, init_db_filename))

      if not retcode == 0:
        os.unlink(init_db_filename)
        logging.error("Failed to init the database. Got error:\n%s" % output)
        sys.exit(1)
      else:
        os.unlink(init_db_filename)

def execute_sql(sql):
  global _mysql_user
  global _mysql_pass
  global _mysql_db_name
  global _mysql_host

  # We create a temporary file in the directory and stream the supplied
  # SQL from that file. This gets us around a dependency on MySQLdb for now
  filename= "sqlfile%d" % random.randint(0,100000)
  f= open(filename, "w+b")
  f.write(sql)
  f.flush()
  (retcode, output)= commands.getstatusoutput("mysql --user=%s --password=%s --host=%s %s < %s" % (_mysql_user, _mysql_pass, _mysql_host, _mysql_db_name, filename))

  f.close()
  if not retcode == 0:
    logging.error("SQL Execution error executing SQL:\n%s\n%s" % (sql, output))
    os.unlink(filename)
    sys.exit(1)
  os.unlink(filename)

def get_select(sql):
  global _mysql_user
  global _mysql_pass
  global _mysql_db_name
  global _mysql_host

  (retcode, output)= commands.getstatusoutput("mysql --user=%s --password=%s --host=%s %s -e\"%s\"" % (_mysql_user, _mysql_pass, _mysql_host, _mysql_db_name, sql))
  if not retcode == 0:
    logging.error("SQL Execution error executing SQL:\n%s\n%s" % (sql, output))
    sys.exit(1)
  # First line is the fields...
  lines= output.split("\n")
  result= []
  fields= lines[0].split("\t")
  lineno= 0
  for line in lines[1:]:
    data_fields= line.split("\t")
    cur_data= data_fields
    result.append(cur_data)
      
  return result

def create_tables():
  create_sql= """
CREATE TABLE sloc_stats (
  server VARCHAR(20) NOT NULL
, version VARCHAR(50) NOT NULL
, directory VARCHAR(50) NOT NULL
, language VARCHAR(20) NOT NULL
, run_date DATETIME NOT NULL
, count INT NOT NULL
, PRIMARY KEY (server, version, directory, language)
) ENGINE=InnoDB;

CREATE TABLE lcov_stats (
  server VARCHAR(20) NOT NULL
, version VARCHAR(50) NULL
, run_date DATETIME NOT NULL
, dir_name VARCHAR(255) NOT NULL
, coverage_percent DECIMAL(5,2) NOT NULL
, PRIMARY KEY (server, version, run_date, dir_name)
) ENGINE=InnoDB;

CREATE TABLE sysbench_config (
  config_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY
, name VARCHAR(255) NOT NULL
) ENGINE=InnoDB;

CREATE TABLE sysbench_runs (
  run_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY
, config_id INT NOT NULL
, server VARCHAR(20) NOT NULL
, version VARCHAR(50) NULL
, run_date DATETIME NOT NULL
) ENGINE=InnoDB;

CREATE TABLE sysbench_run_iterations (
  run_id INT NOT NULL
, concurrency INT NOT NULL
, iteration INT NOT NULL
, tps DECIMAL(13,2) NOT NULL
, read_write_req_per_second DECIMAL(13,2) NOT NULL
, deadlocks_per_second DECIMAL(5,2) NOT NULL
, min_req_latency_ms DECIMAL(10,2) NOT NULL
, avg_req_latency_ms DECIMAL(10,2) NOT NULL
, max_req_latency_ms DECIMAL(10,2) NOT NULL
, 95p_req_latency_ms DECIMAL(10,2) NOT NULL
, PRIMARY KEY (run_id, concurrency, iteration)
) ENGINE=InnoDB;

CREATE TABLE sysbench_run_show_status (
  run_id INT NOT NULL
, concurrency INT NOT NULL
, variable_name VARCHAR(255) NOT NULL
, value INT NOT NULL
, PRIMARY KEY (run_id, concurrency)
) ENGINE=InnoDB;
  
"""
  execute_sql(create_sql)
