#! /usr/bin/python
# -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
# vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
#
# Copyright (C) 2009 Sun Microsystems
#
# Authors:
#
#  Jay Pipes <joinfu@sun.com>
#  Lee Bieber <lee.bieber@sun.com>
#
# This file is part of the Drizzle Automation Project.
#
# The Drizzle Automation Project is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# The Drizzle Automation Project is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with The Drizzle Automation Project (see COPYING.LESSER). If not, 
# see <http://www.gnu.org/licenses/>.

"""Just some useful functions"""

import textwrap
import types
from drizzle.automation.lib import logging

def pretty_print_configuration(configuration, key=None, indent_level= 0):
  """Prints configuration variables for the run."""
  if key:
    print "%s%s: " % ("  "*indent_level, key)
  if type(configuration) is types.DictType:
    for conf_key in configuration.keys():
      pretty_print_configuration(configuration[conf_key], conf_key, indent_level+1)
  else:
    print textwrap.fill("%s%s" % ("  "*(indent_level+1), configuration))

def mail(sender='', to='', subject='', text=''):
  """
  Usage:
  mail('somemailserver.com', 'me@example.com', 'someone@example.com', 'test', 'This is a test')
  """
  headers = "From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" % (sender, to, subject)
  message = headers + text
  import smtplib
  mailServer = smtplib.SMTP('localhost')
  mailServer.sendmail(sender, to, message)
  mailServer.quit()

def drop_and_create_test_database(client):
  """Prepares the benchmark run.  A client instance is passed to prepare()
in order for the instance to create its necessary schema."""

  logging.info("Dropping any test database previously setup.")
  if not client.execute("DROP DATABASE IF EXISTS test"):
    return False

  logging.info("Creating test database.")
  if not client.execute("CREATE DATABASE test"):
    return False
    
  return True

def log_sysbench_run(run_id, config_id, server_name, server_version, run_date):
  """Creates a new run record in the database for this run"""
  sql= """INSERT INTO sysbench_runs (
          run_id
        , config_id
        , server
        , version
        , run_date
        ) VALUES (%d, %d, '%s', '%s', '%s')
      """ % (
        run_id
      , config_id
      , server_name
      , server_version
      , run_date)
  from drizzle.automation.lib import db
  
  db.execute_sql(sql)

def getConfigId(config_name):
  """Returns the integer ID of the configuration name used in this run."""

  # If we have not already done so, we query the local DB for the ID
  # matching this sqlbench config name.  If none is there, we insert
  # a new record in the sysbench_config table and return the newly generated
  # identifier.

  sql= "SELECT config_id FROM sysbench_config WHERE name = '%s'" % config_name

  from drizzle.automation.lib import db
  result= db.get_select(sql)
    
  if len(result) == 0:
    # Insert a new record for this config and return the new ID...
    sql= "INSERT INTO sysbench_config (config_id, name) VALUES (NULL, '%s')" % config_name
    db.execute_sql(sql)
    return getConfigId(config_name)
  else:
    config_id= int(result[0][0])

  return config_id

def getNextRunId():
  """Returns a new run identifier from the database.  
     The run ID is used in logging the results of the run iterations."""
  sql= "SELECT MAX(run_id) as new_run_id FROM sysbench_runs"

  from drizzle.automation.lib import db
  result= db.get_select(sql)
  if result[0][0] == "NULL":
    new_run_id= 1
  else:
    new_run_id= int(result[0][0]) + 1

  return new_run_id
