#! /usr/bin/python
# -*- mode: python; c-basic-offset: 2; indent-tabs-mode: nil; -*-
# vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
#
# Copyright (C) 2009 Sun Microsystems
#
# Authors:
#
#  Jay Pipes <joinfu@sun.com>
#
# This file is part of the Drizzle Automation Project.
#
# The Drizzle Automation Project is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# The Drizzle Automation Project is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with The Drizzle Automation Project (see COPYING.LESSER). If not, 
# see <http://www.gnu.org/licenses/>.

"""Script to automate running of reports."""

import sys
import socket
import os
import os.path
from drizzle.automation.lib import logging
from drizzle.automation.lib import util
import commands
import re
import time
import datetime

def execute(report_name, variables):
  # Verify configuration variables and run the requested report

  if report_name == 'sysbench':
    # We need a config file, a server, a revision, and a branch
    try:
      sysbench_config= variables['sysbench_config']
      bzr_revision= variables['bzr_revision']
      bzr_branch= variables['bzr_branch']
    except KeyError:
      logging.error("The sysbench report requires the --sysbench-config, --bzr-revision, --bzr-branch CLI options to be supplied. Exiting.")
      return False

    try:
      server_name= variables['server']
    except KeyError:
      server_name= 'drizzled'

    import drizzle.automation.reports.sysbench as reports

    # OK, now we need to find the specific last run_id which 
    # corresponds to the revision and branch ...
    run_id= reports.getLastRunId(sysbench_config, server_name, variables['bzr_branch'], int(variables['bzr_revision']))

    if run_id is None:
      logging.error("Could not find the last run ID for config %s, server %s, branch %s at revision %s. Exiting." % (sysbench_config,
                                                                                                                     server_name,
                                                                                                                     variables['bzr_branch'],
                                                                                                                     variables['bzr_revision']))
      return False

    report_text= reports.getSysbenchRegressionReportForRunId(run_id, sysbench_config, server_name, variables['bzr_branch'], int(variables['bzr_revision']))

    if variables['with_email_report'] is True:
      from_string= ('%s <drizzle-benchmark@lists.launchpad.net>' % socket.gethostname())
      util.mail(from_string, variables['sysbench']['report_email'], "SYSBENCH Regression Report for Run ID %d" % int(run_id), report_text)
    else:
      print report_text

    return True
