#! /usr/bin/python
# -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
# vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
#
# Copyright (C) 2009 Sun Microsystems
#
# Authors:
#
#  Jay Pipes <joinfu@sun.com>
#  Lee Bieber <lee.bieber@sun.com>
#
# This file is part of the Drizzle Automation Project.
#
# The Drizzle Automation Project is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# The Drizzle Automation Project is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with The Drizzle Automation Project (see COPYING.LESSER). If not, 
# see <http://www.gnu.org/licenses/>.

"""Various reports on sqlbench command results"""

import os
from drizzle.automation.lib import db
import socket
import commands

def getSysbenchRegressionReport(working_dir, run_id, run_date, server_name, bzr_branch, bzr_revision,storage_engine):
  """Returns a textual report of the regression over a series of runs"""

  # Find the revision comment from BZR
  os.chdir(working_dir)
  (retcode, rev_comment_output)= commands.getstatusoutput("bzr log -r-1 -n0 --line")

  # Output from above command looks like this:
  # jpipes@serialcoder:~/repos/drizzle/trunk-sysbench-r1046$ bzr log -r-1 -n0 --line
  # 1046: Brian Aker 2009-05-31 [merge] Merge Jay.
  # 1039.2.9: Jay Pipes 2009-05-31 Tiny cleanups
  # 1039.2.8: Jay Pipes 2009-05-31 Yet more indentation and style cleanup
  # 1039.2.7: Jay Pipes 2009-05-31 Yet more style and indentation cleanups.
  # 1039.2.6: Jay Pipes 2009-05-31 No code changes...only indentation and style cleanup.
  # 1039.2.5: Jay Pipes 2009-05-31 Style cleanups and moves JOIN_TAB definition out into its own header.
  # 1039.2.4: Jay Pipes 2009-05-31 Tiny indentation cleanup.
  # 1039.2.3: Jay Pipes 2009-05-31 Phase 3 of refactoring JOIN
  # 1039.2.2: Jay Pipes 2009-05-31 Phase 2 of JOIN refactoring.

  comment_lines= rev_comment_output.split("\n")
  rev_comment= comment_lines[0]
  if len(comment_lines) > 1:
    full_commentary= "\n".join(comment_lines[1:])
  else:
    full_commentary= None

  rev_range= range(int(bzr_revision) - 3, int(bzr_revision) + 1)
  revision_versions= ""
  for rev in rev_range:
    revision_versions= revision_versions + "'" + bzr_branch + '-' + str(rev) + "',"
  # Cut off the last comma...
  revision_versions= revision_versions[0:-1]

  sql= """SELECT r.version, r.run_id, r.run_date, i.operation_name, i.seconds, i.usr, i.sys, i.cpu
    FROM sysbench_config c 
    NATURAL JOIN sysbench_runs r
    NATURAL JOIN sqlbench_run_iterations i
    WHERE r.server = '%s'
    AND i.engine = '%s'
    AND r.version IN (%s)
    GROUP BY i.operation_name, r.version
    ORDER BY i.operation_name, r.version
     """ % (
        server_name
      , storage_engine
      , revision_versions
      )

  report_text= """==================================================================
SQLBENCH REGRESSION REPORT
==================================================================
ENGINE:   %s
MACHINE:  %s
RUN ID:   %d
RUN DATE: %s
VERSION:  %s
REVISION: %d
COMMENT:  %s
==================================================================
Version\t\tRun ID\tDate\t\t\tTest\t\tSeconds\tUsr\tSys\tCPU
==================================================================
""" % (
    storage_engine
  , socket.gethostname()
  , run_id
  , run_date
  , bzr_branch
  , int(bzr_revision)
  , rev_comment
)

  results= db.get_select(sql)
  for result in results:
    report_text= report_text + "\t".join(result) + "\n"
  report_text= report_text + "=================================================================="
  if full_commentary:
    report_text= report_text + """
FULL REVISION COMMENTARY:

%s""" % full_commentary
  return report_text
